//             TECHNOLOGICAL ARTS
//              +1(416) 963-8996 
//	    www.technologicalarts.ca

// This is a demo program used to demonstrate the working IIC module 
// of 9S12. In this demo program, we used our stand card Adapt9S12D,9S12
// demo board and IIC PCF8593 Real Time Clock interfaced to H1 connector of Adapt9S12D.
// H1 connector pin 38,39 can be used as SDA and SCL respectively for IIC. Also pin 20
// as IRQ.


// Make sure you initialise the PLL or intialise IIC Frequency Divider Register
// according to the Crytal frequency.

#include <hidef.h>      
#include <mc9s12dp512.h> 
    
#pragma LINK_INFO DERIVATIVE "mc9s12dp512"
#include <PLL.h>
#include <PORT.h>
#include <SRTC.h>
#include <SCI.h>

#define ADD 0x00

#define Set 1
#define Reset 0
#define pagelength 8       // page length 8
#define Read_EEPROM 0x01   


#define EEPROM_ADD 0xA2    // PCF8593 Address

void IIC_read(void);
void Arrange_PCF8593_TO_VB(void);       
void Arrange_VB_TO_PCF8593(void);



unsigned char Data1[8]={0x00,0x00,0x23,0x34,0x86,0xD2,0xF1,0x00};
unsigned char Data2[8];				// Data1 holds the data to be written into a page 
									            // Data2 hold the data when we read the page
unsigned char Data3[8];       // Temporay buffer
unsigned char Data4[8];       // Temporay buffer


int i,n,k,l;
int transmit_flag,write_flag;
 
void main(void) 
{
PLL_Initialization();		// Initialise PLL
PORT_Init();		 	      // Initialise PORT
IIC_Initialization();		// Initialise IIC
SCI_initialization();

INTCR_IRQE=Set;	 			// IRQ interrupt on falling edge
INTCR_IRQEN=Set; 			// enable IRQ interrupt

EnableInterrupts;			// enable interrupts by setting I of CR 

IIC_write(EEPROM_ADD,ADD,Data1,pagelength);  // Initailise PCF8593 with values in Data1
    
  for(;;) 
  {  
 
    if(write_flag==1)              // flag set when user want to change time
    {
      write_flag=0;
      Arrange_VB_TO_PCF8593();  // arrange data in PCF8593 format
      
      IIC_write(EEPROM_ADD,ADD,Data1,pagelength);  // write to PCF8593
    }

    if (transmit_flag==1)         // every second this flag set and send 
	  {                             // present values of PCF8593
		  transmit_flag=0;
		   
      IIC_read();		              // read PCF 8593
		
		  Arrange_PCF8593_TO_VB();    // arrange data in computer format   
 
      for(l=0;l<8;l++)                                                                                       
   		  {
   		    OutSCI(Data4[l]) ;			// send PCF8593 information to computer
   			}
	  }


  }

}



// Recevie Data Interrupt subroutine
#pragma CODE_SEG __NEAR_SEG NON_BANKED 
interrupt void SCI1_ISR(void) 
{
unsigned char READ; 			// READ is just to clear interrupt flag
READ=SCI1SR1; 					  // Clear Interrupt Flag
Data1[n]=HEX_TO_BCD(SCI1DRL);
n++;
if(n==8)					
	{ n=0;
    write_flag=1;         // Write flag set when there is change in time
  }
}

#pragma CODE_SEG DEFAULT


// Recevie Data Interrupt subroutine
#pragma CODE_SEG __NEAR_SEG NON_BANKED 
interrupt void IRQ_ISR(void) 
{  
	transmit_flag=1;        // flag set every second and controller send 
}                         // present time to computer through serial port
#pragma CODE_SEG DEFAULT


//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
// Read the PAGE from EEPROM 24C64.This subroutine require EEPROM device address,
// Location MSB Address and LSB Address.
//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
void IIC_read(void)
{
unsigned char RD_data ;		// for dummy read
int j;        				    // byte count counter

IBCR_TXAK = Reset; 			  // TXAK = 0;
IBCR_TX_RX=Set; 			    // RX/TX = 1; MS/SL = 1; TXAK = 0;
IBCR_MS_SL=Set;				    // And generate START condition;
 
IBDR = EEPROM_ADD; 			  // Address the slave and set up for master transmit;
IIC_test_flag();			    // Test IBIF

IBDR =ADD;			 		      // Send high byte of word address;
IIC_test_flag();			    // Test IBIF

IBCR_RSTA = Set; 				  // set up repeated start;
IBDR = EEPROM_ADD|Read_EEPROM; 	// (slave_address) | (RW = 1);

IIC_test_flag();          // Test IBIF
 

IBCR_TX_RX = Reset; 	  	// set up to receive;
IBCR_TXAK = Reset; 			  // acknowledge disable;
 
for(j=0;j<pagelength;j++)	// this loop will read full page from memory location
{

RD_data = IBDR; 			    // dummy read;
while (!IBSR_IBIF); 		  // wait until IBIF;
IBSR_IBIF=Set; 				    // clear the interrupt event flag;
Data2[j]=IBDR;
}

IBCR_MS_SL = Reset; 		  // generate stop signal;
}




void Arrange_PCF8593_TO_VB(void)
{
 Data4[0]=BCD_TO_HEX(Data2[2]);	// byte 0 second
 Data4[1]=BCD_TO_HEX(Data2[3]);	// byte 1 minute
 
 Data2[2]=Data2[4]&0x3F;				// separate hour info
 Data4[2]=BCD_TO_HEX(Data2[2]);
 
 Data2[0]=Data2[4]&0x40;				// separate AMPM
 Data2[1]=Data2[0]>>6;
 Data4[7]=BCD_TO_HEX(Data2[1]);	// store AMPM, byte 7
 
 Data2[0]=Data2[6]&0xE0;				// separate Day
 Data2[1]=Data2[0]>>5;
 Data4[3]=BCD_TO_HEX(Data2[1]);	// byte 3
 
 Data2[0]=Data2[6]&0x1F;        // separate month
 Data4[5]=BCD_TO_HEX(Data2[0]);
 
 
 Data2[0]=Data2[5]&0x3F;				// separate Date
 Data4[4]=BCD_TO_HEX(Data2[0]);			// byte 4
  
 Data2[0]=Data2[5]&0xC0;				// separate year
 Data2[1]=Data2[0]>>6;					// byte 6
 Data4[6]=BCD_TO_HEX(Data2[1]);
}

void Arrange_VB_TO_PCF8593(void)
{ 
 
  Data3[0]=0x00;        // control word
  Data3[1]=0x00;        // 100th of second
  Data3[2]=Data1[0];    // second
  Data3[3]=Data1[1];    // minute
  
  Data3[4]=Data1[2]|0x80;	// embbed 12/24hr information
	if (Data1[7]==0x01) 	// embed AMPM information into hr byte
	 	Data3[4]|=0x40;
	else
	 	Data3[4]&=0xBF;     // 12/24hr+AM/PM+HOUR
	 
	 
  Data3[5]=Data1[6]<<6; //adjust year
	Data3[5]|=Data1[4];   // year+date
	
	Data3[6]=Data1[3]<<5; // adjust weekday
	Data3[6]|=Data1[5];   // weekday+month
	
	for(i=0;i<pagelength;i++)
	Data1[i]=Data3[i];
	
	
}


